@@ -26,8 +26,10 @@ module Language.Haskell.Exts (
     , parseFile
     , parseFileWithMode
     , parseFileWithExts
+    , parseFileWithComments
     , parseFileContents
     , parseFileContentsWithMode
+    , parseFileContentsWithComments
     -- * Read extensions declared in LANGUAGE pragmas
     , readExtensions
     ) where
@@ -56,6 +58,10 @@ parseFileWithExts exts fp = parseFileWithMode (defaultParseMode { extensions = e
 parseFileWithMode :: ParseMode -> FilePath -> IO (ParseResult Module)
 parseFileWithMode p fp = readFile fp >>= (return . parseFileContentsWithMode p)
 
+-- | Parse a source file on disk, supplying a custom parse mode, and retaining comments.
+parseFileWithComments :: ParseMode -> FilePath -> IO (ParseResult (Module, [Comment]))
+parseFileWithComments p fp = readFile fp >>= (return . parseFileContentsWithComments p)
+
 -- | Parse a source file from a string using the default parse mode.
 parseFileContents :: String -> ParseResult Module
 parseFileContents = parseFileContentsWithMode defaultParseMode
@@ -67,13 +73,22 @@ parseFileContentsWithExts exts = parseFileContentsWithMode (defaultParseMode { e
 
 -- | Parse a source file from a string using a custom parse mode.
 parseFileContentsWithMode :: ParseMode -> String -> ParseResult Module
-parseFileContentsWithMode p@(ParseMode fn exts ign _) rawStr =
+parseFileContentsWithMode p@(ParseMode fn exts ign _ _) rawStr =
         let md = delit fn $ ppContents rawStr
             allExts = impliesExts $ case (ign, readExtensions md) of
                                      (False,Just es) -> exts ++ es
                                      _               -> exts
          in parseWithMode (p { extensions = allExts }) md
 
+-- | Parse a source file from a string using a custom parse mode and retaining comments.
+parseFileContentsWithComments :: ParseMode -> String -> ParseResult (Module, [Comment])
+parseFileContentsWithComments p@(ParseMode fn exts ign _ _) rawStr =
+        let md = delit fn $ ppContents rawStr
+            allExts = impliesExts $ case (ign, readExtensions md) of
+                                     (False,Just es) -> exts ++ es
+                                     _               -> exts
+         in parseWithComments (p { extensions = allExts }) md
+
 
 -- | Gather the extensions declared in LANGUAGE pragmas
 --   at the top of the file. Returns 'Nothing' if the
@@ -82,7 +97,7 @@ readExtensions :: String -> Maybe [Extension]
 readExtensions str = case getTopPragmas str of
         ParseOk pgms -> Just (concatMap getExts pgms)
         _            -> Nothing
-  where getExts :: OptionPragma -> [Extension]
+  where getExts :: ModulePragma -> [Extension]
         getExts (LanguagePragma _ ns) = map readExt ns
         getExts _ = []
 
