@@ -33,6 +33,13 @@ module Language.Haskell.Exts.Annotated (
     , parseFileContentsWithMode
     , parseFileContentsWithExts
     , parseFileContentsWithComments
+    -- * Parsing of Haskell source elements,
+    , parseModule, parseModuleWithMode, parseModuleWithComments
+    , parseExp, parseExpWithMode, parseExpWithComments
+    , parseStmt, parseStmtWithMode, parseStmtWithComments
+    , parsePat, parsePatWithMode, parsePatWithComments
+    , parseDecl, parseDeclWithMode, parseDeclWithComments
+    , parseType, parseTypeWithMode, parseTypeWithComments
     -- * Read extensions declared in LANGUAGE pragmas
     , readExtensions
     ) where
@@ -79,7 +86,7 @@ parseFileContentsWithExts exts = parseFileContentsWithMode (defaultParseMode { e
 
 -- | Parse a source file from a string using a custom parse mode.
 parseFileContentsWithMode :: ParseMode -> String -> ParseResult (Module SrcSpanInfo)
-parseFileContentsWithMode p@(ParseMode fn exts ign _) rawStr =
+parseFileContentsWithMode p@(ParseMode fn exts ign _ _) rawStr =
         let md = delit fn $ ppContents rawStr
             allExts = impliesExts $ case (ign, readExtensions md) of
                                      (False,Just es) -> exts ++ es
@@ -87,7 +94,7 @@ parseFileContentsWithMode p@(ParseMode fn exts ign _) rawStr =
          in parseModuleWithMode (p { extensions = allExts }) md
 
 parseFileContentsWithComments :: ParseMode -> String -> ParseResult (Module SrcSpanInfo, [Comment])
-parseFileContentsWithComments p@(ParseMode fn exts ign _) rawStr =
+parseFileContentsWithComments p@(ParseMode fn exts ign _ _) rawStr =
         let md = delit fn $ ppContents rawStr
             allExts = impliesExts $ case (ign, readExtensions md) of
                                      (False,Just es) -> exts ++ es
@@ -101,7 +108,7 @@ readExtensions :: String -> Maybe [Extension]
 readExtensions str = case getTopPragmas str of
         ParseOk pgms -> Just (concatMap getExts pgms)
         _            -> Nothing
-  where getExts :: OptionPragma l -> [Extension]
+  where getExts :: ModulePragma l -> [Extension]
         getExts (LanguagePragma _ ns) = map readExt ns
         getExts _ = []
 
