@@ -18,7 +18,7 @@
 -- ToDo: FloatTok should have three parts (integer part, fraction, exponent) (?)
 -- ToDo: Use a lexical analyser generator (lx?)
 
-module Language.Haskell.Exts.Lexer (Token(..), lexer) where
+module Language.Haskell.Exts.Lexer (Token(..), showToken, lexer) where
 
 import Language.Haskell.Exts.ParseMonad
 import Language.Haskell.Exts.SrcLoc
@@ -28,6 +28,7 @@ import Language.Haskell.Exts.ExtScheme
 
 import Data.Char
 import Data.Ratio
+import Data.List (intersperse)
 import Control.Monad (when)
 
 -- import Debug.Trace (trace)
@@ -126,9 +127,9 @@ data Token
 -- Pragmas
 
         | PragmaEnd                     -- #-}
---        | PragmaUnknown (String,String)   -- Any pragma not recognized
         | RULES
         | INLINE Bool
+        | INLINE_CONLIKE
         | SPECIALISE
         | SPECIALISE_INLINE Bool
         | SOURCE
@@ -139,11 +140,10 @@ data Token
         | CORE
         | UNPACK
         | OPTIONS (Maybe String,String)
-        | CFILES  String
+--        | CFILES  String
+--        | INCLUDE String
         | LANGUAGE
-        | INCLUDE String
--- These are not yet implemented
---        | LINE
+        | ANN
 
 -- Reserved Ids
 
@@ -211,7 +211,16 @@ reserved_ops = [
  ( "-<",  (LeftArrowTail,       Just (Any [Arrows])) ),
  ( ">-",  (RightArrowTail,      Just (Any [Arrows])) ),
  ( "-<<", (LeftDblArrowTail,    Just (Any [Arrows])) ),
- ( ">>-", (RightDblArrowTail,   Just (Any [Arrows])) )
+ ( ">>-", (RightDblArrowTail,   Just (Any [Arrows])) ),
+ -- Unicode notation
+ ( "\x2190",    (LeftArrow,     Just (Any  [UnicodeSyntax])) ),
+ ( "\x2192",    (RightArrow,    Just (Any  [UnicodeSyntax])) ),
+ ( "\x21d2",    (DoubleArrow,   Just (Any  [UnicodeSyntax])) ),
+ ( "\x2919",    (LeftArrowTail,     Just (All [UnicodeSyntax, Arrows])) ),
+ ( "\x291a",    (RightArrowTail,    Just (All [UnicodeSyntax, Arrows])) ),
+ ( "\x291b",    (LeftDblArrowTail,  Just (All [UnicodeSyntax, Arrows])) ),
+ ( "\x291c",    (RightDblArrowTail, Just (All [UnicodeSyntax, Arrows])) ),
+ ( "\x2605",    (Star,              Just (All [UnicodeSyntax, KindSignatures])) )
  ]
 
 special_varops :: [(String,(Token, Maybe ExtScheme))]
@@ -256,7 +265,10 @@ reserved_ids = [
  ( "where",     (KW_Where,      Nothing) ),
 
 -- FFI
- ( "foreign",   (KW_Foreign,    Just (Any [ForeignFunctionInterface])) )
+ ( "foreign",   (KW_Foreign,    Just (Any [ForeignFunctionInterface])) ),
+
+-- Unicode
+ ( "\x2200",    (KW_Forall,     Just (All [UnicodeSyntax, ExplicitForall])) )
  ]
 
 
@@ -286,14 +298,15 @@ pragmas = [
  ( "source",            SOURCE          ),
  ( "deprecated",        DEPRECATED      ),
  ( "warning",           WARNING         ),
+ ( "ann",               ANN             ),
  ( "scc",               SCC             ),
  ( "generated",         GENERATED       ),
  ( "core",              CORE            ),
  ( "unpack",            UNPACK          ),
  ( "language",          LANGUAGE        ),
- ( "options",           OPTIONS undefined ), -- we'll tweak it before use - promise!
- ( "cfiles",            CFILES  undefined ), -- same here...
- ( "include",           INCLUDE undefined )  -- ...and here!
+ ( "options",           OPTIONS undefined ) -- we'll tweak it before use - promise!
+-- ( "cfiles",            CFILES  undefined ), -- same here...
+-- ( "include",           INCLUDE undefined )  -- ...and here!
  ]
 
 isIdent, isHSymbol :: Char -> Bool
@@ -339,9 +352,15 @@ topLexer = do
 lexWhiteSpace :: Bool -> Lex a (Bool, Bool)
 lexWhiteSpace bol = do
     s <- getInput
+    ignL <- ignoreLinePragmasL
     case s of
         -- If we find a recognised pragma, we don't want to treat it as a comment.
         '{':'-':'#':rest | isRecognisedPragma rest -> return (bol, False)
+                         | isLinePragma rest && not ignL -> do 
+                            (l, fn) <- lexLinePragma
+                            setSrcLineL l
+                            setLineFilenameL fn
+                            lexWhiteSpace True
         '{':'-':_ -> do
             loc <- getSrcLocL
             discard 2
@@ -379,12 +398,33 @@ lexWhiteSpace bol = do
             return (bol, True)
         _ -> return (bol, False)
 
-isRecognisedPragma :: String -> Bool
+isRecognisedPragma, isLinePragma :: String -> Bool
 isRecognisedPragma str = let pragma = map toLower . takeWhile isAlphaNum . dropWhile isSpace $ str
                           in case lookup pragma pragmas of
                               Nothing -> False
                               _       -> True
 
+isLinePragma str = let pragma = map toLower . takeWhile isAlphaNum . dropWhile isSpace $ str
+                    in case pragma of
+                        "line"  -> True
+                        _       -> False
+
+lexLinePragma :: Lex a (Int, String)
+lexLinePragma = do
+    discard 3   -- {-#
+    lexWhile isSpace
+    discard 4   -- LINE
+    lexWhile isSpace
+    i <- lexWhile isDigit
+    lexWhile isSpace
+    matchChar '"' "Improperly formatted LINE pragma"
+    fn <- lexWhile (/= '"')
+    matchChar '"' "Impossible - lexLinePragma"
+    lexWhile isSpace
+    mapM (flip matchChar "Improperly formatted LINE pragma") "#-}"
+    lexNewline
+    return (read i, fn)
+
 lexNestedComment :: Bool -> String -> Lex a (Bool, String)
 lexNestedComment bol str = do
     s <- getInput
@@ -705,12 +745,26 @@ lexStdToken = do
             lexQQBody = do
                 s <- getInput
                 case s of
-                  '\\':']':_ -> do str <- lexQQBody
+                  '\\':']':_ -> do discard 2
+                                   str <- lexQQBody
                                    return (']':str)
-                  '\\':'|':_ -> do str <- lexQQBody
+                  '\\':'|':_ -> do discard 2
+                                   str <- lexQQBody
                                    return ('|':str)
                   '|':']':_  -> discard 2 >> return ""
-                  _ -> do str <- lexWhile (not . (`elem` "\\|"))
+                  '|':_ -> do discard 1
+                              str <- lexQQBody
+                              return ('|':str)
+                  ']':_ -> do discard 1
+                              str <- lexQQBody
+                              return (']':str)
+                  '\\':_ -> do discard 1
+                               str <- lexQQBody
+                               return ('\\':str)
+                  '\n':_ -> do lexNewline
+                               str <- lexQQBody
+                               return ('\n':str)
+                  _ -> do str <- lexWhile (not . (`elem` "\\|\n"))
                           rest <- lexQQBody
                           return (str++rest)
 
@@ -719,6 +773,13 @@ lexPragmaStart = do
     lexWhile isSpace
     pr <- lexWhile isAlphaNum
     case lookup (map toLower pr) pragmas of
+     Just (INLINE True) -> do
+            s <- getInput
+            case map toLower s of
+             '_':'c':'o':'n':'l':'i':'k':'e':_  -> do
+                      discard 8
+                      return $ INLINE_CONLIKE
+             _ -> return $ INLINE True
      Just SPECIALISE -> do
             s <- getInput
             case dropWhile isSpace $ map toLower s of
@@ -748,12 +809,12 @@ lexPragmaStart = do
                 rest <- lexRawPragma
                 return $ OPTIONS (Nothing, rest)
              _ -> fail "Malformed Options pragma"
-     Just (CFILES _) -> do
+{-     Just (CFILES _) -> do
             rest <- lexRawPragma
             return $ CFILES rest
      Just (INCLUDE _) -> do
             rest <- lexRawPragma
-            return $ INCLUDE rest
+            return $ INCLUDE rest -}
      Just p ->  return p
 
      _      -> fail "Internal error: Unrecognised recognised pragma"
@@ -856,8 +917,8 @@ lexConIdOrQual qual = do
                                _ -> return ident
                     case lookup ident' reserved_ids of
                        -- cannot qualify a reserved word
-                       Just _  -> just_a_conid
-                       Nothing -> return (QVarId (qual', ident'))
+                       Just (_,scheme) | isEnabled scheme exts  -> just_a_conid
+                       _ -> return (QVarId (qual', ident'))
 
              | isUpper c -> do      -- qualified conid?
                     discard 1
@@ -1064,3 +1125,145 @@ parseInteger radix ds =
 
 flagKW :: Token -> Lex a ()
 flagKW t = when (t `elem` [KW_Do, KW_MDo]) flagDo
+
+------------------------------------------------------------------
+-- "Pretty" printing for tokens
+
+showToken :: Token -> String
+showToken t = case t of
+  VarId s           -> s
+  QVarId (q,s)      -> q ++ '.':s
+  IDupVarId s       -> '?':s
+  ILinVarId s       -> '%':s
+  ConId s           -> s
+  QConId (q,s)      -> q ++ '.':s
+  DVarId ss         -> concat $ intersperse "-" ss
+  VarSym s          -> s
+  ConSym s          -> s
+  QVarSym (q,s)     -> q ++ '.':s
+  QConSym (q,s)     -> q ++ '.':s
+  IntTok (_, s)         -> s
+  FloatTok (_, s)       -> s
+  Character (_, s)      -> '\'':s ++ "'"
+  StringTok (_, s)      -> '"':s ++ "\""
+  IntTokHash (_, s)     -> s ++ "#"
+  WordTokHash (_, s)    -> s ++ "##"
+  FloatTokHash (_, s)   -> s ++ "#"
+  DoubleTokHash (_, s)  -> s ++ "##"
+  CharacterHash (_, s)  -> '\'':s ++ "'#"
+  StringHash (_, s)     -> '"':s ++ "\"#"
+  LeftParen         -> "("
+  RightParen        -> ")"
+  LeftHashParen     -> "(#"
+  RightHashParen    -> "#)"
+  LeftCurlyBar      -> "{|"
+  RightCurlyBar     -> "|}"
+  SemiColon         -> ";"
+  LeftCurly         -> "{"
+  RightCurly        -> "}"
+  VRightCurly       -> "virtual }"
+  LeftSquare        -> "["
+  RightSquare       -> "]"
+  Comma             -> ","
+  Underscore        -> "_"
+  BackQuote         -> "`"
+  Dot               -> "."
+  DotDot            -> ".."
+  Colon             -> ":"
+  DoubleColon       -> "::"
+  Equals            -> "="
+  Backslash         -> "\\"
+  Bar               -> "|"
+  LeftArrow         -> "->"
+  RightArrow        -> "<-"
+  At                -> "@"
+  Tilde             -> "~"
+  DoubleArrow       -> "=>"
+  Minus             -> "-"
+  Exclamation       -> "!"
+  Star              -> "*"
+  LeftArrowTail     -> ">-"
+  RightArrowTail    -> "-<"
+  LeftDblArrowTail  -> ">>-"
+  RightDblArrowTail -> "-<<"
+  THExpQuote        -> "[|"
+  THPatQuote        -> "[p|"
+  THDecQuote        -> "[d|"
+  THTypQuote        -> "[t|"
+  THCloseQuote      -> "|]"
+  THIdEscape s      -> '$':s
+  THParenEscape     -> "$("
+  THVarQuote        -> "'"
+  THTyQuote         -> "''"
+  THQuasiQuote (n,q) -> "[$" ++ n ++ "|" ++ q ++ "]"
+  RPGuardOpen       -> "(|"
+  RPGuardClose      -> "|)"
+  RPCAt             -> "@:"
+  XCodeTagOpen      -> "<%"
+  XCodeTagClose     -> "%>"
+  XStdTagOpen       -> "<"
+  XStdTagClose      -> ">"
+  XCloseTagOpen     -> "</"
+  XEmptyTagClose    -> "/>"
+  XPCDATA s         -> "PCDATA " ++ s
+  XRPatOpen         -> "<["
+  XRPatClose        -> "]>"
+  PragmaEnd         -> "#-}"
+  RULES             -> "{-# RULES"
+  INLINE b          -> "{-# " ++ if b then "INLINE" else "NOINLINE"
+  INLINE_CONLIKE    -> "{-# " ++ "INLINE_CONLIKE"
+  SPECIALISE        -> "{-# SPECIALISE"
+  SPECIALISE_INLINE b -> "{-# SPECIALISE " ++ if b then "INLINE" else "NOINLINE"
+  SOURCE            -> "{-# SOURCE"
+  DEPRECATED        -> "{-# DEPRECATED"
+  WARNING           -> "{-# WARNING"
+  SCC               -> "{-# SCC"
+  GENERATED         -> "{-# GENERATED"
+  CORE              -> "{-# CORE"
+  UNPACK            -> "{-# UNPACK"
+  OPTIONS (mt,s)    -> "{-# OPTIONS" ++ maybe "" (':':) mt ++ " ..."
+--  CFILES  s         -> "{-# CFILES ..."
+--  INCLUDE s         -> "{-# INCLUDE ..."
+  LANGUAGE          -> "{-# LANGUAGE"
+  ANN               -> "{-# ANN"
+  KW_As         -> "as"
+  KW_By         -> "by"
+  KW_Case       -> "case"
+  KW_Class      -> "class"
+  KW_Data       -> "data"
+  KW_Default    -> "default"
+  KW_Deriving   -> "deriving"
+  KW_Do         -> "do"
+  KW_MDo        -> "mdo"
+  KW_Else       -> "else"
+  KW_Family     -> "family"
+  KW_Forall     -> "forall"
+  KW_Group      -> "group"
+  KW_Hiding     -> "hiding"
+  KW_If         -> "if"
+  KW_Import     -> "import"
+  KW_In         -> "in"
+  KW_Infix      -> "infix"
+  KW_InfixL     -> "infixl"
+  KW_InfixR     -> "infixr"
+  KW_Instance   -> "instance"
+  KW_Let        -> "let"
+  KW_Module     -> "module"
+  KW_NewType    -> "newtype"
+  KW_Of         -> "of"
+  KW_Proc       -> "proc"
+  KW_Rec        -> "rec"
+  KW_Then       -> "then"
+  KW_Type       -> "type"
+  KW_Using      -> "using"
+  KW_Where      -> "where"
+  KW_Qualified  -> "qualified"
+  KW_Foreign    -> "foreign"
+  KW_Export     -> "export"
+  KW_Safe       -> "safe"
+  KW_Unsafe     -> "unsafe"
+  KW_Threadsafe -> "threadsafe"
+  KW_StdCall    -> "stdcall"
+  KW_CCall      -> "ccall"
+
+  EOF           -> "EOF"
