@@ -321,7 +321,7 @@ checkPat e [] = case e of
                         _ -> patFail ""
             _ -> patFail ""
     TupleSection l mes    ->
-            if not (any ((=~=) Nothing) mes)
+            if not (any ((==) Nothing) mes)
              then do ps <- mapM (\e -> checkPat e []) (map fromJust mes)
                      return (PTuple l ps)
              else fail "Illegal tuple section in pattern"
@@ -528,7 +528,7 @@ checkExpr e = case e of
                      return (S.Case l e alts)
     Do l stmts            -> checkDo stmts >> return (S.Do l stmts)
     MDo l stmts           -> checkDo stmts >> return (S.MDo l stmts)
-    TupleSection l mes -> if not (any ((=~=) Nothing) mes)
+    TupleSection l mes -> if not (any ((==) Nothing) mes)
                            then checkManyExprs (map fromJust mes) (S.Tuple l)
                            else do checkEnabled TupleSections
                                    mes' <- mapM mCheckExpr mes
@@ -665,13 +665,16 @@ getGConName _ = fail "Expression in reification is not a name"
 checkValDef :: L -> PExp L -> Maybe (S.Type L) -> Rhs L -> Maybe (Binds L) -> P (Decl L)
 checkValDef l lhs optsig rhs whereBinds = do
     mlhs <- isFunLhs lhs []
+    let whpt = srcInfoPoints l
     case mlhs of
-     Just (f,es,b) -> do
+     Just (f,es,b,pts) -> do
             ps <- mapM checkPattern es
+            let l' = l { srcInfoPoints = pts ++ whpt }
             case optsig of -- only pattern bindings can have signatures
                 Nothing -> return (FunBind l $
-                            if b then [Match l f ps rhs whereBinds]
-                                 else let [a,b] = ps in [InfixMatch l a f b rhs whereBinds])
+                            if b then [Match l' f ps rhs whereBinds]
+                                 else let (a:bs) = ps 
+                                       in [InfixMatch l' a f bs rhs whereBinds])
                 Just _  -> fail "Cannot give an explicit type signature to a function binding"
      Nothing     -> do
             lhs <- checkPattern lhs
@@ -679,19 +682,24 @@ checkValDef l lhs optsig rhs whereBinds = do
 
 -- A variable binding is parsed as a PatBind.
 
-isFunLhs :: PExp L -> [PExp L] -> P (Maybe (Name L, [PExp L], Bool))
-isFunLhs (InfixApp _ l (QVarOp loc (UnQual _ op)) r) es
+isFunLhs :: PExp L -> [PExp L] -> P (Maybe (Name L, [PExp L], Bool, [S]))
+isFunLhs (InfixApp ll l (QVarOp loc (UnQual _ op)) r) es
     | op =~= (Symbol () "!") = do
         exts <- getExtensions
         if BangPatterns `elem` exts
          then let (b,bs) = splitBang r []
                in isFunLhs l (BangPat loc b : bs ++ es)
-         else return $ Just (op, l:r:es, False) -- It's actually a definition of the operator !
-    | otherwise = return $ Just (op, l:r:es, False)
-isFunLhs (App _ (Var _ (UnQual _ f)) e) es = return $ Just (f, e:es, True)
+         else return $ Just (op, l:r:es, False, []) -- It's actually a definition of the operator !
+    | otherwise = return $ Just (op, l:r:es, False, [])
+isFunLhs (App _ (Var _ (UnQual _ f)) e) es = return $ Just (f, e:es, True, [])
 isFunLhs (App _ f e) es = isFunLhs f (e:es)
-isFunLhs (Var _ (UnQual _ f)) es@(_:_) = return $ Just (f, es, True)
-isFunLhs (Paren _ f) es@(_:_) = isFunLhs f es
+isFunLhs (Var _ (UnQual _ f)) es@(_:_) = return $ Just (f, es, True, [])
+isFunLhs (Paren l f) es@(_:_) = do mlhs <- isFunLhs f es 
+                                   case mlhs of
+                                    Just (f,es,b,pts) -> 
+                                       let [x,y] = srcInfoPoints l
+                                        in return $ Just (f,es,b,x:pts++[y])
+                                    _ -> return Nothing
 isFunLhs _ _ = return Nothing
 
 -- Separating between signature declarations and value definitions in
@@ -735,17 +743,16 @@ checkUnQual (Special _ _) = fail "Illegal special name"
 
 -----------------------------------------------------------------------------
 -- Check that two xml tag names are equal
--- Could use Eq directly, but I am not sure whether <dom:name>...</name>
--- would be valid, in that case Eq won't work. TODO
-
 checkEqNames :: XName L -> XName L -> P (XName L)
 checkEqNames n@(XName _ n1) (XName _ n2)
     | n1 == n2  = return n
-    | otherwise = fail "names in matching xml tags are not equal"
 checkEqNames n@(XDomName _ d1 n1) (XDomName _ d2 n2)
     | n1 == n2 && d1 == d2 = return n
-    | otherwise = fail "names in matching xml tags are not equal"
-checkEqNames _ _ = fail "names in matching xml tags are not equal"
+checkEqNames n m = fail $ "opening tag '" ++ showTag n ++
+                   "' does not match closing tag '" ++ showTag m ++ "'"
+    where
+        showTag (XName _ n) = n
+        showTag (XDomName _ d n) = d ++ ":" ++ n
 
 
 -----------------------------------------------------------------------------
@@ -779,6 +786,13 @@ checkRevDecls = mergeFunBinds []
                     -- `atSrcLoc` loc
             else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeFunBinds (FunBind l ms':revDs) ds
+    mergeFunBinds revDs (FunBind l ims1@(InfixMatch _ _ name _ _ _:_):ds1) =
+        mergeInfix ims1 ds1 l
+        where
+        mergeInfix ims' (FunBind _ ims@(InfixMatch loc _ name' _ _ _:_):ds) l
+            | name' =~= name =
+            mergeInfix (ims++ims') ds (loc <++> l)
+        mergeInfix ms' ds l = mergeFunBinds (FunBind l ms':revDs) ds
     mergeFunBinds revDs (d:ds) = mergeFunBinds (d:revDs) ds
 
 checkRevClsDecls :: [ClassDecl L] -> P [ClassDecl L]
@@ -796,6 +810,13 @@ checkRevClsDecls = mergeClsFunBinds []
                     -- `atSrcLoc` loc
             else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeClsFunBinds (ClsDecl l (FunBind l ms'):revDs) ds
+    mergeClsFunBinds revDs (ClsDecl l (FunBind _ ims1@(InfixMatch _ _ name _ _ _:_)):ds1) =
+        mergeInfix ims1 ds1 l
+        where
+        mergeInfix ims' (ClsDecl _ (FunBind _ ims@(InfixMatch loc _ name' _ _ _:_)):ds) l
+            | name' =~= name =
+            mergeInfix (ims++ims') ds (loc <++> l)
+        mergeInfix ms' ds l = mergeClsFunBinds (ClsDecl l (FunBind l ms'):revDs) ds
     mergeClsFunBinds revDs (d:ds) = mergeClsFunBinds (d:revDs) ds
 
 checkRevInstDecls :: [InstDecl L] -> P [InstDecl L]
@@ -814,6 +835,13 @@ checkRevInstDecls = mergeInstFunBinds []
                     -- `atSrcLoc` loc
             else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeInstFunBinds (InsDecl l (FunBind l ms'):revDs) ds
+    mergeInstFunBinds revDs (InsDecl l (FunBind _ ims1@(InfixMatch _ _ name _ _ _:_)):ds1) =
+        mergeInfix ims1 ds1 l
+        where
+        mergeInfix ims' (InsDecl _ (FunBind _ ims@(InfixMatch loc _ name' _ _ _:_)):ds) l
+            | name' =~= name =
+            mergeInfix (ims++ims') ds (loc <++> l)
+        mergeInfix ms' ds l = mergeInstFunBinds (InsDecl l (FunBind l ms'):revDs) ds
     mergeInstFunBinds revDs (d:ds) = mergeInstFunBinds (d:revDs) ds
 
 ----------------------------------------------------------------
@@ -873,7 +901,7 @@ checkTypes = mapM (flip checkT True)
 ---------------------------------------
 -- Converting a complete page
 
-checkPageModule :: PExp L -> ([OptionPragma L],[S],L) -> P (Module L)
+checkPageModule :: PExp L -> ([ModulePragma L],[S],L) -> P (Module L)
 checkPageModule xml (os,ss,inf) = do
     mod <- getModuleName
     xml <- checkExpr xml
@@ -890,485 +918,16 @@ checkHybridModule xml (Module inf mh os is ds) s1 s2 = do
         S.XETag l xn ats mattr    -> return $ XmlHybrid (inf<++>l<**(s1 : srcInfoPoints inf ++ s2 : srcInfoPoints l))
                                                 mh os is ds xn ats mattr []
 
-{-
-pageFun :: L -> S.Exp L -> Decl L
-pageFun loc e = PatBind loc namePat Nothing rhs Nothing
-    where namePat = PVar loc $ Ident loc "page"
-          rhs = UnGuardedRhs (ann e) e
-
-mkPage :: Module L -> L -> S.Exp L -> P (Module L)
-mkPage (Module src mmh os imps decls) loc xml = do
-    let page = pageFun loc xml
-    return $ Module src mmh os imps (decls ++ [page])
-
-mkPageModule :: [OptionPragma L] -> S.Exp L -> P (Module L)
-mkPageModule os xml = do
-    do loc <- case xml of
-           S.XTag l _ _ _ _ -> return l
-           S.XETag l _ _ _  -> return l
-           _ -> fail "Will not happen since mkPageModule is only called on XML expressions"
-       mod <- getModuleName
-       return $ (Module
-              loc
-              (ModuleName loc mod)
-              os
-              Nothing
-              (Just (ExportSpecList loc [EVar loc $ UnQual loc $ Ident loc "page"]))
-              []
-              [pageFun loc xml])
--}
 ---------------------------------------
 -- Handle dash-identifiers
 
 mkDVar :: [String] -> String
 mkDVar = concat . intersperse "-"
 
---mkDVarExpr :: L -> [String] -> PExp L
---mkDVarExpr l = foldl1 (\x y -> InfixApp l x (op $ sym "-") y) . map (Var l . UnQual l . name)
-
 ---------------------------------------
 -- Combine adjacent for-alls. NO!
 --
 -- A valid type must have one for-all at the top of the type, or of the fn arg types
 
 mkTyForall :: L -> Maybe [TyVarBind L] -> Maybe (PContext L) -> PType L -> PType L
---mkTyForall l mtvs (PContext _ [])   ty = mk_forall_ty l mtvs ty
 mkTyForall l mtvs ctxt ty = TyForall l mtvs ctxt ty
-
-{-- mk_forall_ty makes a pure for-all type (no context)
-mk_forall_ty l (Just []) ty             = ty  -- Explicit for-all with no tyvars
-mk_forall_ty l mtvs1     (TyForall _ mtvs2 ctxt ty) = mkTyForall l (mtvs1 `plus` mtvs2) ctxt ty
-mk_forall_ty l mtvs1     ty             = TyForall l mtvs1 (PContext l []) ty
-
-mtvs1       `plus` Nothing     = mtvs1
-Nothing     `plus` mtvs2       = mtvs2
-(Just tvs1) `plus` (Just tvs2) = Just (tvs1 ++ tvs2)
--}
----------------------------------------
-{-- Expressions as we parse them (and patters, and regular patterns)
-
-data PExp l
-    = Var l (QName l)                 -- ^ variable
-    | IPVar l (IPName l)              -- ^ implicit parameter variable
-    | Con l (QName l)                 -- ^ data constructor
-    | Lit l (Literal l)               -- ^ literal constant
-    | InfixApp l (PExp l) (QOp l) (PExp l)    -- ^ infix application
-    | App l (PExp l) (PExp l)             -- ^ ordinary application
-    | NegApp l (PExp l)               -- ^ negation expression @-@ /exp/
-    | Lambda l [Pat l] (PExp l) -- ^ lambda expression
-    | Let l (Binds l) (PExp l)           -- ^ local declarations with @let@
-    | If l (PExp l) (PExp l) (PExp l)         -- ^ @if@ /exp/ @then@ /exp/ @else@ /exp/
-    | Case l (PExp l) [Alt l]           -- ^ @case@ /exp/ @of@ /alts/
-    | Do l [Stmt l]                 -- ^ @do@-expression:
-                                    -- the last statement in the list
-                                    -- should be an expression.
-    | MDo l [Stmt l]                -- ^ @mdo@-expression
---    | Tuple [PExp]              -- ^ tuple expression
-    | TupleSection l [Maybe (PExp l)] -- ^ tuple section expression, e.g. @(,,3)@
-    | List l [PExp l]               -- ^ list expression
-    | Paren l (PExp l)                -- ^ parenthesized expression
---     RightSection QOp PExp     -- ^ right section @(@/qop/ /exp/@)@
-    | RecConstr l (QName l) [PFieldUpdate l]
-                                -- ^ record construction expression
-    | RecUpdate l (PExp l) [PFieldUpdate l]
-                                -- ^ record update expression
-    | EnumFrom l (PExp l)             -- ^ unbounded arithmetic sequence,
-                                    -- incrementing by 1
-    | EnumFromTo l (PExp l) (PExp l)      -- ^ bounded arithmetic sequence,
-                                    -- incrementing by 1
-    | EnumFromThen l (PExp l) (PExp l)   -- ^ unbounded arithmetic sequence,
-                                    -- with first two elements given
-    | EnumFromThenTo l (PExp l) (PExp l) (PExp l)
-                                -- ^ bounded arithmetic sequence,
-                                    -- with first two elements given
-    | ParComp l (PExp l) [[QualStmt l]]    -- ^ parallel list comprehension
-    | ExpTypeSig l (PExp l) (S.Type l)
-                                -- ^ expression type signature
-    | AsPat l (Name l) (PExp l)           -- ^ patterns only
-    | WildCard l                 -- ^ patterns only
-    | IrrPat l (PExp l)               -- ^ patterns only
-
--- Post-ops for parsing left sections and regular patterns. Not to be left in the final tree.
-    | PostOp l (PExp l) (QOp l)          -- ^ post-ops
-    | PreOp l (QOp l) (PExp l)            -- ^ pre-ops
-
--- View patterns
-    | ViewPat l (PExp l) (PExp l)         -- ^ patterns only
-
--- HaRP
-    | SeqRP l [PExp l]              -- ^ regular patterns only
-    | GuardRP l (PExp l) [Stmt l]       -- ^ regular patterns only
-    | EitherRP l (PExp l) (PExp l)        -- ^ regular patterns only
-    | CAsRP l (Name l) (PExp l)           -- ^ regular patterns only
-
--- Template Haskell
-    | VarQuote l (QName l)            -- ^ 'x
-    | TypQuote l (QName l)            -- ^ ''T
-    | BracketExp l (Bracket l)
-    | SpliceExp l (Splice l)
-    | QuasiQuote l String String  -- ^ [$...|...]
-
--- Hsx
-    | XTag  l (XName l) [ParseXAttr l] (Maybe (PExp l)) [PExp l]
-    | XETag l (XName l) [ParseXAttr l] (Maybe (PExp l))
-    | XPcdata l String
-    | XExpTag l (PExp l)
-    | XRPats l [PExp l]
-
--- Pragmas
-    | CorePragma l      String  (PExp l)
-    | SCCPragma  l      String  (PExp l)
-    | GenPragma  l      String (Int, Int) (Int, Int) (PExp l)
---    | UnknownExpPragma  String String
-
--- Generics
-    | ExplTypeArg l (QName l) (S.Type l)   -- ^ f {| Int |} x = ...
-
--- Bang Patterns
-    | BangPat l (PExp l)              -- ^ f !a = ...
-
--- Arrows
-    | Proc l (Pat l) (PExp l)
-    | LeftArrApp      l (PExp l) (PExp l)
-    | RightArrApp     l (PExp l) (PExp l)
-    | LeftArrHighApp  l (PExp l) (PExp l)
-    | RightArrHighApp l (PExp l) (PExp l)
-   deriving (Eq,Show)
-
-data PFieldUpdate l
-    = FieldUpdate l (QName l) (PExp l)
-    | FieldPun l (Name l)
-    | FieldWildcard l
-  deriving (Eq,Show)
-
-data ParseXAttr l = XAttr l (XName l) (PExp l)
-  deriving (Eq,Show)
-
-instance Annotated PExp where
-    ann e = case e of
-        Var l qn        -> l
-        IPVar l ipn     -> l
-        Con l qn        -> l
-        Lit l lit       -> l
-        InfixApp l e1 qop e2    -> l
-        App l e1 e2     -> l
-        NegApp l e      -> l
-        Lambda l ps e   -> l
-        Let l bs e      -> l
-        If l ec et ee   -> l
-        Case l e alts   -> l
-        Do l ss         -> l
-        MDo l ss        -> l
-        TupleSection l mes  -> l
-        List l es       -> l
-        Paren l e       -> l
-        RecConstr l qn fups     -> l
-        RecUpdate l e  fups     -> l
-        EnumFrom l e            -> l
-        EnumFromTo l ef et      -> l
-        EnumFromThen l ef et    -> l
-        EnumFromThenTo l ef eth eto -> l
-        ParComp  l e qsss       -> l
-        ExpTypeSig l e t        -> l
-        AsPat l n e             -> l
-        WildCard l              -> l
-        IrrPat l e              -> l
-        PostOp l e op           -> l
-        PreOp l op e            -> l
-        ViewPat l e1 e2         -> l
-        SeqRP l es              -> l
-        GuardRP l e ss          -> l
-        EitherRP l e1 e2        -> l
-        CAsRP l n e             -> l
-
-        VarQuote l qn           -> l
-        TypQuote l qn           -> l
-        BracketExp l br         -> l
-        SpliceExp l sp          -> l
-        QuasiQuote l sn se      -> l
-
-        XTag  l xn xas me es    -> l
-        XETag l xn xas me       -> l
-        XPcdata l s             -> l
-        XExpTag l e             -> l
-        XRPats l es             -> l
-
-        CorePragma l s e   -> l
-        SCCPragma  l s e   -> l
-        GenPragma  l s n12 n34 e -> l
-
-        ExplTypeArg l qn t      -> l
-        BangPat l e             -> l
-
-        Proc            l p e   -> l
-        LeftArrApp      l e1 e2 -> l
-        RightArrApp     l e1 e2 -> l
-        LeftArrHighApp  l e1 e2 -> l
-        RightArrHighApp l e1 e2 -> l
-
-    amap f e = case e of
-        Var l qn                -> Var   (f l) qn
-        IPVar l ipn             -> IPVar (f l) ipn
-        Con l qn                -> Con   (f l) qn
-        Lit l lit               -> Lit   (f l) lit
-        InfixApp l e1 qop e2    -> InfixApp (f l) e1 qop e2
-        App l e1 e2             -> App (f l) e1 e2
-        NegApp l e              -> NegApp (f l) e
-        Lambda l ps e           -> Lambda (f l) ps e
-        Let l bs e              -> Let (f l) bs e
-        If l ec et ee           -> If (f l) ec et ee
-        Case l e alts           -> Case (f l) e alts
-        Do l ss                 -> Do (f l) ss
-        MDo l ss                -> MDo (f l) ss
-        TupleSection l mes      -> TupleSection (f l) mes
-        List l es               -> List (f l) es
-        Paren l e               -> Paren (f l) e
-        RecConstr l qn fups     -> RecConstr (f l) qn fups
-        RecUpdate l e  fups     -> RecUpdate (f l) e  fups
-        EnumFrom l e            -> EnumFrom (f l) e
-        EnumFromTo l ef et      -> EnumFromTo (f l) ef et
-        EnumFromThen l ef et    -> EnumFromThen (f l) ef et
-        EnumFromThenTo l ef eth eto -> EnumFromThenTo (f l) ef eth eto
-        ParComp  l e qsss       -> ParComp  (f l) e qsss
-        ExpTypeSig l e t        -> ExpTypeSig (f l) e t
-
-        AsPat l n e             -> AsPat (f l) n e
-        WildCard l              -> WildCard (f l)
-        IrrPat l e              -> IrrPat (f l) e
-        PostOp l e op           -> PostOp (f l) e op
-        PreOp l op e            -> PreOp (f l) op e
-        ViewPat l e1 e2         -> ViewPat (f l) e1 e2
-        SeqRP l es              -> SeqRP (f l) es
-        GuardRP l e ss          -> GuardRP (f l) e ss
-        EitherRP l e1 e2        -> EitherRP (f l) e1 e2
-        CAsRP l n e             -> CAsRP (f l) n e
-        ExplTypeArg l n t       -> ExplTypeArg (f l) n t
-        BangPat l e             -> BangPat (f l) e
-
-        VarQuote l qn           -> VarQuote (f l) qn
-        TypQuote l qn           -> TypQuote (f l) qn
-        BracketExp l br         -> BracketExp (f l) br
-        SpliceExp l sp          -> SpliceExp (f l) sp
-        QuasiQuote l sn se      -> QuasiQuote (f l) sn se
-
-        XTag  l xn xas me es    -> XTag  (f l) xn xas me es
-        XETag l xn xas me       -> XETag (f l) xn xas me
-        XPcdata l s             -> XPcdata (f l) s
-        XExpTag l e             -> XExpTag (f l) e
-
-        CorePragma l s e        -> CorePragma (f l) s e
-        SCCPragma  l s e        -> SCCPragma  (f l) s e
-        GenPragma  l s n12 n34 e -> GenPragma  (f l) s n12 n34 e
-
-        Proc            l p e   -> Proc            (f l) p e
-        LeftArrApp      l e1 e2 -> LeftArrApp      (f l) e1 e2
-        RightArrApp     l e1 e2 -> RightArrApp     (f l) e1 e2
-        LeftArrHighApp  l e1 e2 -> LeftArrHighApp  (f l) e1 e2
-        RightArrHighApp l e1 e2 -> RightArrHighApp (f l) e1 e2
-
-instance Functor PExp where
-      fmap f e = case e of
-          Var l qn                -> Var   (f l) (fmap f qn)
-          IPVar l ipn             -> IPVar (f l) (fmap f ipn)
-          Con l qn                -> Con   (f l) (fmap f qn)
-          Lit l lit               -> Lit   (f l) (fmap f lit)
-          InfixApp l e1 qop e2    -> InfixApp (f l) (fmap f e1) (fmap f qop) (fmap f e2)
-          App l e1 e2             -> App (f l) (fmap f e1) (fmap f e2)
-          NegApp l e              -> NegApp (f l) (fmap f e)
-          Lambda l ps e           -> Lambda (f l) (map (fmap f) ps) (fmap f e)
-          Let l bs e              -> Let (f l) (fmap f bs) (fmap f e)
-          If l ec et ee           -> If (f l) (fmap f ec) (fmap f et) (fmap f ee)
-          Case l e alts           -> Case (f l) (fmap f e) (map (fmap f) alts)
-          Do l ss                 -> Do (f l) (map (fmap f) ss)
-          MDo l ss                -> MDo (f l) (map (fmap f) ss)
-          TupleSection l mes      -> TupleSection (f l) (map (fmap (fmap f)) mes)
-          List l es               -> List (f l) (map (fmap f) es)
-          Paren l e               -> Paren (f l) (fmap f e)
-          RecConstr l qn fups     -> RecConstr (f l) (fmap f qn) (map (fmap f) fups)
-          RecUpdate l e  fups     -> RecUpdate (f l) (fmap f e)  (map (fmap f) fups)
-          EnumFrom l e            -> EnumFrom (f l) (fmap f e)
-          EnumFromTo l ef et      -> EnumFromTo (f l) (fmap f ef) (fmap f et)
-          EnumFromThen l ef et    -> EnumFromThen (f l) (fmap f ef) (fmap f et)
-          EnumFromThenTo l ef eth eto -> EnumFromThenTo (f l) (fmap f ef) (fmap f eth) (fmap f eto)
-          ParComp  l e qsss       -> ParComp  (f l) (fmap f e) (map (map (fmap f)) qsss)
-          ExpTypeSig l e t        -> ExpTypeSig (f l) (fmap f e) (fmap f t)
-
-          AsPat l n e             -> AsPat (f l) (fmap f n) (fmap f e)
-          WildCard l              -> WildCard (f l)
-          IrrPat l e              -> IrrPat (f l) (fmap f e)
-          PostOp l e op           -> PostOp (f l) (fmap f e) (fmap f op)
-          PreOp l op e            -> PreOp (f l) (fmap f op) (fmap f e)
-          ViewPat l e1 e2         -> ViewPat (f l) (fmap f e1) (fmap f e2)
-          SeqRP l es              -> SeqRP (f l) (map (fmap f) es)
-          GuardRP l e ss          -> GuardRP (f l) (fmap f e) (map (fmap f) ss)
-          EitherRP l e1 e2        -> EitherRP (f l) (fmap f e1) (fmap f e2)
-          CAsRP l n e             -> CAsRP (f l) (fmap f n) (fmap f e)
-          ExplTypeArg l n t       -> ExplTypeArg (f l) (fmap f n) (fmap f t)
-          BangPat l e             -> BangPat (f l) (fmap f e)
-
-          VarQuote l qn           -> VarQuote (f l) (fmap f qn)
-          TypQuote l qn           -> TypQuote (f l) (fmap f qn)
-          BracketExp l br         -> BracketExp (f l) (fmap f br)
-          SpliceExp l sp          -> SpliceExp (f l) (fmap f sp)
-          QuasiQuote l sn se      -> QuasiQuote (f l) sn se
-
-          XTag  l xn xas me es    -> XTag  (f l) (fmap f xn) (map (fmap f) xas) (fmap (fmap f) me) (map (fmap f) es)
-          XETag l xn xas me       -> XETag (f l) (fmap f xn) (map (fmap f) xas) (fmap (fmap f) me)
-          XPcdata l s             -> XPcdata (f l) s
-          XExpTag l e             -> XExpTag (f l) (fmap f e)
-
-          CorePragma l s e        -> CorePragma (f l) s (fmap f e)
-          SCCPragma  l s e        -> SCCPragma  (f l) s (fmap f e)
-          GenPragma  l s n12 n34 e -> GenPragma  (f l) s n12 n34 (fmap f e)
-
-          Proc            l p e   -> Proc            (f l) (fmap f p) (fmap f e)
-          LeftArrApp      l e1 e2 -> LeftArrApp      (f l) (fmap f e1) (fmap f e2)
-          RightArrApp     l e1 e2 -> RightArrApp     (f l) (fmap f e1) (fmap f e2)
-          LeftArrHighApp  l e1 e2 -> LeftArrHighApp  (f l) (fmap f e1) (fmap f e2)
-          RightArrHighApp l e1 e2 -> RightArrHighApp (f l) (fmap f e1) (fmap f e2)
-
-
-
-instance Functor PFieldUpdate where
-    fmap f (FieldUpdate l qn e) = FieldUpdate (f l) (fmap f qn) (fmap f e)
-    fmap f (FieldPun l n)       = FieldPun (f l) (fmap f n)
-    fmap f (FieldWildcard l)    = FieldWildcard (f l)
-
-instance Annotated PFieldUpdate where
-    ann (FieldUpdate l qn e) = l
-    ann (FieldPun l n)       = l
-    ann (FieldWildcard l)    = l
-    amap f (FieldUpdate l qn e) = FieldUpdate (f l) qn e
-    amap f (FieldPun l n)       = FieldPun (f l) n
-    amap f (FieldWildcard l)    = FieldWildcard (f l)
-
-instance Functor ParseXAttr where
-    fmap f (XAttr l xn e) = XAttr (f l) (fmap f xn) (fmap f e)
-
-instance Annotated ParseXAttr where
-    ann (XAttr l _ _) = l
-    amap f (XAttr l xn e) = XAttr (f l) xn e
-
-p_unit_con :: l -> PExp l
-p_unit_con l         = Con l (unit_con_name l)
-
-p_tuple_con :: l -> Boxed -> Int -> PExp l
-p_tuple_con l b i       = Con l (tuple_con_name l b i)
-
-p_unboxed_singleton_con :: l -> PExp l
-p_unboxed_singleton_con l = Con l (unboxed_singleton_con_name l)
-
-data PContext l
-    = CxSingle l (PAsst l)
-    | CxTuple  l [PAsst l]
-    | CxParen  l (PContext l)
-    | CxEmpty  l
- deriving (Eq, Show)
-
-instance Functor PContext where
-  fmap f (CxSingle l asst) = CxSingle (f l) (fmap f asst)
-  fmap f (CxTuple l assts) = CxTuple (f l) (map (fmap f) assts)
-  fmap f (CxParen l ctxt)  = CxParen (f l) (fmap f ctxt)
-  fmap f (CxEmpty l)       = CxEmpty (f l)
-
-instance Annotated PContext where
-  ann (CxSingle l asst ) = l
-  ann (CxTuple  l assts) = l
-  ann (CxParen  l ctxt ) = l
-  ann (CxEmpty  l)       = l
-  amap f (CxSingle l asst ) = CxSingle (f l) asst
-  amap f (CxTuple  l assts) = CxTuple  (f l) assts
-  amap f (CxParen  l ctxt ) = CxParen  (f l) ctxt
-  amap f (CxEmpty l) = CxEmpty (f l)
-
-data PType l
-     = TyForall l
-        (Maybe [TyVarBind l])
-        (Maybe (PContext l))
-        (PType l)
-     | TyFun   l (PType l) (PType l)            -- ^ function type
-     | TyTuple l Boxed     [PType l]            -- ^ tuple type, possibly boxed
-     | TyList  l (PType l)                      -- ^ list syntax, e.g. [a], as opposed to [] a
-     | TyApp   l (PType l) (PType l)            -- ^ application of a type constructor
-     | TyVar   l (Name l)                       -- ^ type variable
-     | TyCon   l (QName l)                      -- ^ named type or type constructor
-     | TyParen l (PType l)                      -- ^ type surrounded by parentheses
-     | TyPred  l (PAsst l)                      -- ^ assertion of an implicit parameter
-     | TyInfix l (PType l) (QName l) (PType l)  -- ^ infix type constructor
-     | TyKind  l (PType l) (Kind l)             -- ^ type with explicit kind signature
-  deriving (Eq, Show)
-
-instance Functor PType where
-    fmap f t = case t of
-      TyForall l mtvs mcx t         -> TyForall (f l) (fmap (map (fmap f)) mtvs) (fmap (fmap f) mcx) (fmap f t)
-      TyFun   l t1 t2               -> TyFun (f l) (fmap f t1) (fmap f t2)
-      TyTuple l b ts                -> TyTuple (f l) b (map (fmap f) ts)
-      TyList  l t                   -> TyList (f l) (fmap f t)
-      TyApp   l t1 t2               -> TyApp (f l) (fmap f t1) (fmap f t2)
-      TyVar   l n                   -> TyVar (f l) (fmap f n)
-      TyCon   l qn                  -> TyCon (f l) (fmap f qn)
-      TyParen l t                   -> TyParen (f l) (fmap f t)
-      TyPred  l asst                -> TyPred (f l) (fmap f asst)
-      TyInfix l ta qn tb            -> TyInfix (f l) (fmap f ta) (fmap f qn) (fmap f tb)
-      TyKind  l t k                 -> TyKind (f l) (fmap f t) (fmap f k)
-
-instance Annotated PType where
-    ann t = case t of
-      TyForall l mtvs cx t          -> l
-      TyFun   l t1 t2               -> l
-      TyTuple l b ts                -> l
-      TyList  l t                   -> l
-      TyApp   l t1 t2               -> l
-      TyVar   l n                   -> l
-      TyCon   l qn                  -> l
-      TyParen l t                   -> l
-      TyInfix l ta qn tb            -> l
-      TyKind  l t k                 -> l
-    amap f t = case t of
-      TyForall l mtvs mcx t         -> TyForall (f l) mtvs mcx t
-      TyFun   l t1 t2               -> TyFun (f l) t1 t2
-      TyTuple l b ts                -> TyTuple (f l) b ts
-      TyList  l t                   -> TyList (f l) t
-      TyApp   l t1 t2               -> TyApp (f l) t1 t2
-      TyVar   l n                   -> TyVar (f l) n
-      TyCon   l qn                  -> TyCon (f l) qn
-      TyParen l t                   -> TyParen (f l) t
-      TyInfix l ta qn tb            -> TyInfix (f l) ta qn tb
-      TyKind  l t k                 -> TyKind (f l) t k
-
-data PAsst l
-    = ClassA l (QName l) [PType l]
-    | InfixA l (PType l) (QName l) (PType l)
-    | IParam l (IPName l) (PType l)
-    | EqualP l (PType l)  (PType l)
-  deriving (Eq, Show)
-
-instance Functor PAsst where
-    fmap f asst = case asst of
-        ClassA l qn ts      -> ClassA (f l) (fmap f qn) (map (fmap f) ts)
-        InfixA l ta qn tb   -> InfixA (f l) (fmap f ta) (fmap f qn) (fmap f tb)
-        IParam l ipn t      -> IParam (f l) (fmap f ipn) (fmap f t)
-        EqualP l t1 t2      -> EqualP (f l) (fmap f t1) (fmap f t2)
-
-instance Annotated PAsst where
-    ann asst = case asst of
-        ClassA l qn ts      -> l
-        InfixA l ta qn tb   -> l
-        IParam l ipn t      -> l
-        EqualP l t1 t2      -> l
-    amap f asst = case asst of
-        ClassA l qn ts      -> ClassA (f l) qn ts
-        InfixA l ta qn tb   -> InfixA (f l) ta qn tb
-        IParam l ipn t      -> IParam (f l) ipn t
-        EqualP l t1 t2      -> EqualP (f l) t1 t2
-
-
-unit_tycon, fun_tycon, list_tycon, unboxed_singleton_tycon :: l -> PType l
-unit_tycon              l = TyCon l (unit_tycon_name l)
-fun_tycon               l = TyCon l (fun_tycon_name l)
-list_tycon              l = TyCon l (list_tycon_name l)
-unboxed_singleton_tycon l = TyCon l (unboxed_singleton_tycon_name l)
-
-tuple_tycon :: l -> Boxed -> Int -> PType l
-tuple_tycon l b i         = TyCon l (tuple_tycon_name l b i)
--}
\ No newline at end of file
